// 002 u_c_analyze_iso_file
// 27 sep 2004

// -- (C) John COLIBRI 2006
// -- documentation: http://www.jcolibri.com

// -- c_iso_image

(*$r+*)

unit u_c_analyze_iso_file;
  interface
    uses u_iso_definitions
        , u_c_iso_file_reader
        , u_c_iso_path_and_file;

    type c_analyze_iso_file= class
                               m_iso_file_name: string;

                               m_c_iso_file_reader: c_iso_file_reader;
                               m_c_iso_path_and_file_ref: c_iso_path;

                               constructor create_analyze_iso_file(p_iso_file_name: string);

                               function f_open_iso_file(p_c_path_file_ref: c_iso_path): Boolean;
                               procedure build_path_file_tree(p_root_directory_record: TRootDirectoryRecord);
                               function f_extract_file(p_c_iso_file: c_iso_file;
                                   p_full_save_name: String): Boolean;

                               destructor Destroy; override;
                             end; // c_analyze_iso_file

  implementation
    uses Math // Min()
        , Classes // TStrings
        , SysUtils
        , u_c_display
        , u_dir;

    type PByte= ^Byte;

    constructor c_analyze_iso_file.create_analyze_iso_file(p_iso_file_name: string);
      begin
        inherited Create;
        m_iso_file_name:= p_iso_file_name;
      end; // create_analyze_iso_file

    function c_analyze_iso_file.f_open_iso_file(p_c_path_file_ref: c_iso_path): Boolean;
      var l_volume_descriptor: TVolumeDescriptor;
          l_root_directory_record: TRootDirectoryRecord;
      begin
        display('> f_open_iso_file');
        Result:= False;
        m_c_iso_path_and_file_ref:= p_c_path_file_ref;

        if (FileExists(m_iso_file_name))
          then
            begin
              m_c_iso_file_reader:= c_iso_file_reader.create_iso_file_handler(m_iso_file_name);
              display('file "'+ m_iso_file_name+ '" opened...');

              // -- skip the first 16 sectors
              m_c_iso_file_reader.f_seek_sector(16);

              if (m_c_iso_file_reader.m_iso_file_format= e_complete_sectors_file_format)
                then display('contains RAW data')
                else
                  if (m_c_iso_file_reader.m_iso_file_format= e_only_data_file_format)
                    then display('contains sector data');

              if (m_c_iso_file_reader.m_iso_yellow_book_format= e_mode_1_format)
                then display('contains yellow book mode 1 data')
                else
                  if (m_c_iso_file_reader.m_iso_yellow_book_format= e_mode_2_format)
                    then display('contains yellow book mode 2 data');

              display('user data sector size is '
                  + IntToStr(m_c_iso_file_reader.f_sector_data_size)+ ' bytes');
              display('data offset in file is '
                  + IntToStr(m_c_iso_file_reader.m_file_offset)+ ' bytes');

              if (m_c_iso_file_reader.f_sector_data_size<> 2048)
                then
                  begin
                    display('sorry, but sector size other than 2048 bytes are not supported');
                    Exit;
                  end;

              repeat
                m_c_iso_file_reader.f_read_sector_data(l_volume_descriptor, SizeOf(TVolumeDescriptor));

                case Byte(l_volume_descriptor.DescriptorType) of
                  vdtPVD: begin
                            display('Primary Volume Descriptor found');
                            l_root_directory_record:= l_volume_descriptor.Primary.RootDirectory;

                            build_path_file_tree(l_root_directory_record);
                          end;
                  vdtSVD: begin
                            display_bug_stop('Supplementary Volume Descriptor found');
                          end;
                  vdtBR: begin
                           // display_bug_stop('Boot Record Volume Descriptor found');
                           display('*** Boot Record Volume Descriptor found');
                         end;
                end; // case
              until (l_volume_descriptor.DescriptorType= vdtVDST);

              Result:= True;
            end
          else display_bug_halt('file '+ m_iso_file_name+ ' not found');

        display('< f_open_iso_file');
      end; // f_open_iso_file

    procedure c_analyze_iso_file.build_path_file_tree(p_root_directory_record: TRootDirectoryRecord);
        // -- called by f_open_iso_image
      var l_c_path_list: tStringList;

      procedure parse_directory_sector_recursive(p_level: Integer; p_end_sector: Cardinal;
          p_path, p_last_segment: String; p_c_iso_path: c_iso_path);
        var l_pt_sector_buffer: PByte;
            l_pt_work_pointer: pByte;

            l_pt_directory_entry: t_pt_directory_entry;
            l_file_name_length: Integer;
            l_file_name: string;
            l_c_iso_path: c_iso_path;

            l_end_sector: Cardinal;
            l_save_stream_position: Integer;
        begin
          display('> build '+ p_path);
          // l_c_path_list.Add(p_path);
          l_c_iso_path:= p_c_iso_path.f_c_get_or_add_sub_path(p_path, p_last_segment);

          // -- allocate and load the whole sector in memory
          GetMem(l_pt_sector_buffer, m_c_iso_file_reader.f_sector_data_size);

          try
            // -- start at the beginning of the sector
            l_pt_work_pointer:= l_pt_sector_buffer;
            // -- load in the buffer (which is also referenced by the work pointer)
            m_c_iso_file_reader.f_read_sector_data(l_pt_work_pointer^, m_c_iso_file_reader.f_sector_data_size);

            while m_c_iso_file_reader.m_current_sector_number<= p_end_sector do
            begin
              // -- a directory cannot be straddled above 2 sectors. If not enough room
              // -- remaining in the sector, read the next
              if m_c_iso_file_reader.f_sector_data_size- (Cardinal(l_pt_work_pointer)- Cardinal(l_pt_sector_buffer))
                      < SizeOf(t_directory_entry)
                then
                  begin
                    l_pt_work_pointer:= l_pt_sector_buffer;
                    m_c_iso_file_reader.f_read_sector_data(l_pt_work_pointer^, m_c_iso_file_reader.f_sector_data_size);
                  end;

              // -- allocate and extract the directory record
              New(l_pt_directory_entry);
              Move(l_pt_work_pointer^, l_pt_directory_entry^, SizeOf(t_directory_entry));

              // -- skip the fixed part
              Inc(l_pt_work_pointer, SizeOf(t_directory_entry));

              // -- extract the file name
              l_file_name_length:= l_pt_directory_entry.LengthOfFileIdentifier;
              if l_file_name_length= 0
                then begin
                    // -- NO: break
                    // -- can have '' as a file name, and still other entries after this name
                    l_file_name:= '';
                  end
                else begin
                    SetLength(l_file_name, l_file_name_length);
                    Move(l_pt_work_pointer^, l_file_name[1], l_file_name_length);
                  end;

              // -- skip the file name
              Inc(l_pt_work_pointer, l_file_name_length);

              // -- skip any padding bytes
              if ((SizeOf(t_directory_entry)+ l_file_name_length)
                  < l_pt_directory_entry.LengthOfDirectoryRecord)
                then Inc(l_pt_work_pointer, l_pt_directory_entry.LengthOfDirectoryRecord
                    - SizeOf(t_directory_entry)- l_file_name_length);

              if (l_pt_directory_entry.FileFlags and $2)= $2
                then begin
                    // -- this is a directory

                    // -- discard names starting with #0 and #1
                    if (l_file_name<> #0) and (l_file_name<> #1)
                      then
                        begin
                          l_save_stream_position:= m_c_iso_file_reader.m_current_sector_number;

                          // -- read the new directory sector
                          m_c_iso_file_reader.f_seek_sector(l_pt_directory_entry.LocationOfExtent.
                              LittleEndian);
                          l_end_sector:= l_pt_directory_entry.LocationOfExtent.LittleEndian
                              + (l_pt_directory_entry.DataLength.LittleEndian
                                 + m_c_iso_file_reader.f_sector_data_size- 1)
                              div m_c_iso_file_reader.f_sector_data_size;

                          parse_directory_sector_recursive(p_level+ 1, l_end_sector,
                              p_path+ l_file_name+ '\', l_file_name, l_c_iso_path);

                          m_c_iso_file_reader.f_seek_sector(l_save_stream_position);
                        end;
                  end
                else begin
                    if (l_file_name<> '') and (l_pt_directory_entry.DataLength.LittleEndian> 0)
                      then
                        // -- create the file with the 2 values useful for extraction
                        with l_pt_directory_entry^ do
                          l_c_iso_path.f_c_add_file(l_file_name,
                              LocationOfExtent.LittleEndian, DataLength.LittleEndian);
                  end;

              Dispose(l_pt_directory_entry);
            end; // while m_c_iso_file_reader.m_current_sector_number<= l_end_sector

          finally
            // -- free the sector memory
            FreeMem(l_pt_sector_buffer, m_c_iso_file_reader.f_sector_data_size);
          end;

          display('< build');
        end; // parse_directory_sector_recursive

      var l_end_sector: Cardinal;

      begin // build_path_file_tree
        display('> build_path_file_tree');

        display('directory sector '
            + IntToStr(p_root_directory_record.LocationOfExtent.LittleEndian));
        l_end_sector:= p_root_directory_record.LocationOfExtent.LittleEndian
            + (p_root_directory_record.DataLength.LittleEndian+ m_c_iso_file_reader.f_sector_data_size- 1)
            div m_c_iso_file_reader.f_sector_data_size;

        m_c_iso_file_reader.f_seek_sector(p_root_directory_record.LocationOfExtent.LittleEndian);

        l_c_path_list:= tStringList.Create;
        parse_directory_sector_recursive(0, l_end_sector, '', '', m_c_iso_path_and_file_ref);

        with l_c_path_list do
        begin
          // display_strings_only(l_c_path_list);
          Free;
        end;

        display('< build_path_file_tree');
      end; // build_path_file_tree

    function c_analyze_iso_file.f_extract_file(p_c_iso_file: c_iso_file;
        p_full_save_name: String): Boolean;
        // -- read a file's data from the ISO file and save it in a DOS file
      var l_path: String;
          l_remaining_bytes: Int64;
          l_c_file_stream: TFileStream;
          l_pt_sector_buffer: Pointer;
      begin
        Result:= False;

        l_path:= ExtractFilePath(p_full_save_name);
        if f_create_path(l_path)
          then display('PATH '+ l_path);

        with p_c_iso_file do
        begin
          m_c_iso_file_reader.f_seek_sector(m_iso_sector_number);
          l_remaining_bytes:= m_iso_size;
        end;

        l_c_file_stream:= TFileStream.Create(p_full_save_name, fmCreate);
        GetMem(l_pt_sector_buffer, m_c_iso_file_reader.f_sector_data_size);

        try
          while (l_remaining_bytes> 0) do
          begin
            m_c_iso_file_reader.f_read_sector_data(l_pt_sector_buffer^, m_c_iso_file_reader.f_sector_data_size);
            l_c_file_stream.WriteBuffer(l_pt_sector_buffer^, Min(l_remaining_bytes, m_c_iso_file_reader.
                f_sector_data_size));
            Dec(l_remaining_bytes, m_c_iso_file_reader.f_sector_data_size);
          end;
          Result:= True;
        finally
          l_c_file_stream.Free;
          FreeMem(l_pt_sector_buffer, m_c_iso_file_reader.f_sector_data_size);
        end;
      end; // f_extract_file

    destructor c_analyze_iso_file.Destroy;
      begin
        if (Assigned(m_c_iso_file_reader))
          then FreeAndNil(m_c_iso_file_reader);

        inherited;
      end; // Destroy

    end.


