// 002 u_c_iso_file_reader
// 27 sep 2004

// -- (C) John COLIBRI 2006
// -- documentation: http://www.jcolibri.com

// -- reads iso sectors
// -- find cd type

(*$r+*)

unit u_c_iso_file_reader;
  interface
    uses Classes
        , SysUtils; // Exception, FileExists()

    type EISOImageHandlerException= class(Exception);

         // -- Yellow Book Format

         // -- e_auto_format   = auto detect
         // -- e_mode_1_format  = 2048 byte user data
         // -- e_mode_2_format  = 2336 byte user data
         t_iso_yellow_book_format_type= (e_auto_format, e_mode_1_format, e_mode_2_format);

         // -- Image Format

         // -- e_complete_sectors_file_format  = with SYNC, Header, etc
         // -- e_only_data_file_format         = only the user data
         t_iso_file_format_type= (e_auto_file_format, e_complete_sectors_file_format, e_only_data_file_format);

         c_iso_file_reader= class
                                m_iso_yellow_book_format: t_iso_yellow_book_format_type;
                                m_iso_file_format: t_iso_file_format_type;
                                m_file_offset: Cardinal;
                                m_c_file_stream: TFileStream;
                                m_current_sector_number: Cardinal;

                                constructor create_iso_file_handler(const p_iso_file_name: string);
                                procedure detect_iso_file_type;
                                function compute_sector_offset(const p_sector_number: Cardinal):
                                      Integer;
                                function f_compute_user_data_offset: Integer;
                                function f_sector_data_size: Cardinal;
                                function f_seek_sector(const p_sector_number: Cardinal;
                                      const p_allow_grow: Boolean= True): Boolean;
                                function f_read_sector_data(var pvu_buffer; const
                                      p_actual_buffer_size: Integer= - 1): Boolean;
                                function f_read_sector_raw(var pvu_buffer; const
                                      p_actual_buffer_size: Integer= - 1): Boolean;
                                destructor Destroy; override;
                            end; // c_iso_file_reader

  implementation

    // -- c_iso_file_reader

    constructor c_iso_file_reader.create_iso_file_handler(const p_iso_file_name: string);
      begin
        inherited Create;
        if (not FileExists(p_iso_file_name))
          then raise EISOImageHandlerException.Create('image file not found');

        m_c_file_stream:= TFileStream.Create(p_iso_file_name,
            fmOpenReadWrite or fmShareDenyNone);
        detect_iso_file_type;
        f_seek_sector(m_current_sector_number);
      end; // create_iso_file_handler

    procedure c_iso_file_reader.detect_iso_file_type;
      type TCheckBuf= packed record
                               DeskID: Byte;
                               StdID: array[0..4] of Char;
                             end;
           TRawCheckBuf= packed record
                                  SYNC: array[0..11] of Byte;
                                  Header_SectMin, Header_SectSec, Header_SectNo,
                                      Header_Mode: Byte;
                                  Deskriptor: TCheckBuf;
                                end;
      var Buff: TCheckBuf;
          RawBuff: TRawCheckBuf;
          Tries: Boolean;
      begin
        // -- defaults
        m_iso_yellow_book_format:= e_auto_format;
        m_iso_file_format:= e_auto_file_format;
        m_file_offset:= 0;

        if (Assigned(m_c_file_stream)) and (m_c_file_stream.Size> 16* 2048)
          then
            begin
              for Tries:= False to True do
              begin
                if (Tries)
                  then // -- ok, 2nd run, last try: nero .nrg image file
                       m_file_offset:= 307200;

                m_c_file_stream.Position:= m_file_offset+ 16* 2048;
                m_c_file_stream.ReadBuffer(Buff, SizeOf(Buff));
                if (string(Buff.StdID)= 'CD001')
                  then
                    begin
                      m_iso_file_format:= e_only_data_file_format;
                      m_iso_yellow_book_format:= e_mode_1_format;
                      Break;
                    end;

                m_c_file_stream.Position:= m_file_offset+ 16* 2336;
                m_c_file_stream.ReadBuffer(Buff, SizeOf(Buff));
                if (string(Buff.StdID)= 'CD001')
                  then
                    begin
                      m_iso_file_format:= e_only_data_file_format;
                      m_iso_yellow_book_format:= e_mode_2_format;
                      Break;
                    end;

                m_c_file_stream.Position:= m_file_offset+ 16* 2352;
                m_c_file_stream.ReadBuffer(RawBuff, SizeOf(RawBuff));
                if (string(RawBuff.Deskriptor.StdID)= 'CD001')
                  then
                    begin
                      m_iso_file_format:= e_complete_sectors_file_format;
                      if (RawBuff.Header_Mode= 1)
                        then m_iso_yellow_book_format:= e_mode_1_format
                        else
                          if (RawBuff.Header_Mode= 2)
                            then m_iso_yellow_book_format:= e_mode_2_format
                            else raise EISOImageHandlerException.Create(
                                'unkown Yellow Book mode!');
                      Break;
                    end;
              end; // for Tries
            end;

        if (m_iso_file_format= e_auto_file_format) or (m_iso_yellow_book_format= e_auto_format)
          then raise EISOImageHandlerException.Create('unkown image format!');
      end; // detect_iso_file_type

    function c_iso_file_reader.f_compute_user_data_offset: Integer;
      begin
        case m_iso_file_format of

          e_complete_sectors_file_format:
            // -- 12 bytes SYNC, 4 byte Header
            Result:= 16;
          e_only_data_file_format: Result:= 0;
          e_auto_file_format: raise EISOImageHandlerException.Create(
                      'can not calculate sector offset with auto values!');
          else raise EISOImageHandlerException.Create(
                'TImageFileHandler.CalcUserDataOffset(): Implementation error!');
        end;
      end; // f_compute_user_data_offset

    function c_iso_file_reader.compute_sector_offset(const p_sector_number: Cardinal): Integer;
      begin
        case m_iso_file_format of
          e_complete_sectors_file_format: Result:= m_file_offset+ p_sector_number* 2352;
          e_only_data_file_format: begin
                        case m_iso_yellow_book_format of
                          e_mode_1_format: Result:= m_file_offset+ p_sector_number* 2048;
                          e_mode_2_format: Result:= m_file_offset+ p_sector_number* 2336;
                          e_auto_format: raise EISOImageHandlerException.Create(
                                       'can not calculate sector with auto settings');
                          else raise EISOImageHandlerException.Create(
                                'TImageFileHandler.CalcSectorOffset(): Implementation error!');
                        end;
                      end;
          e_auto_file_format: raise EISOImageHandlerException.Create(
                      'can not calculate sector with auto settings');
          else raise EISOImageHandlerException.Create(
                'TImageFileHandler.CalcSectorOffset(): Implementation error!');
        end;
      end; // compute_sector_offset

    function c_iso_file_reader.f_seek_sector(const p_sector_number: Cardinal;
        const p_allow_grow: Boolean): Boolean;
      var lFPos: Integer;
      begin
        Result:= False;
        if (Assigned(m_c_file_stream))
          then
            begin
              lFPos:= compute_sector_offset(p_sector_number);
              if ((lFPos+ 2048)> m_c_file_stream.Size) and (not p_allow_grow)
                then Exit;
              m_c_file_stream.Position:= lFPos;
              m_current_sector_number:= p_sector_number;
            end;
      end; // f_seek_sector

    function c_iso_file_reader.f_read_sector_data(var pvu_buffer;
        const p_actual_buffer_size: Integer= - 1): Boolean;
      var lDataOffset: Integer;
      begin
        Result:= False;

        if (Assigned(m_c_file_stream))
          then
            begin
              lDataOffset:= f_compute_user_data_offset;
              m_c_file_stream.Seek(lDataOffset, soFromCurrent);
              if (p_actual_buffer_size> - 1) and (Cardinal(p_actual_buffer_size)< f_sector_data_size)
                then raise EISOImageHandlerException.Create(
                    'TImageFileHandler.ReadSector_Data(): buffer overflow protection');
              m_c_file_stream.ReadBuffer(pvu_buffer, f_sector_data_size);
              f_seek_sector(m_current_sector_number+ 1);
              Result:= True;
            end;
      end; // f_read_sector_data

    function c_iso_file_reader.f_read_sector_raw(var pvu_buffer;
        const p_actual_buffer_size: Integer= - 1): Boolean;
      begin
        Result:= False;
        if (Assigned(m_c_file_stream))
          then
            begin
              if (p_actual_buffer_size> - 1) and (p_actual_buffer_size< 2352)
                then raise EISOImageHandlerException.Create(
                    'TImageFileHandler.ReadSector_Raw(): buffer overflow protection');
              m_c_file_stream.ReadBuffer(pvu_buffer, 2352);
              Result:= True;
            end;
      end; // f_read_sector_raw

    function c_iso_file_reader.f_sector_data_size: Cardinal;
      begin
        case m_iso_yellow_book_format of
          e_mode_1_format: Result:= 2048;
          e_mode_2_format: Result:= 2336;
          else raise EISOImageHandlerException.Create(
                'can not figure out sector data size on auto type');
        end;
      end; // f_sector_data_size

    destructor c_iso_file_reader.Destroy;
      begin
        if (Assigned(m_c_file_stream))
          then FreeAndNil(m_c_file_stream);

        inherited;
      end; // Destroy

    end.

