// 002 u_c_iso_path_and_file
// 27 sep 2004

// -- (C) John COLIBRI 2006
// -- documentation: http://www.jcolibri.com

(*$r+*)

unit u_c_iso_path_and_file;
  interface
    uses Classes, u_c_basic_object;

    type c_iso_file= // one "file"
                     Class(c_basic_object)
                       // -- m_name: the file+ extension name
                       m_iso_sector_number: Integer;
                       m_iso_size: Integer;

                       Constructor create_iso_file(p_name: String;
                           p_iso_sector_number, p_iso_size: Integer);
                       function f_display_file: String;
                       function f_c_self: c_iso_file;
                       Destructor Destroy; Override;
                     end; // c_iso_file

         c_iso_path= // -- recursive structure containing the iso directories
                     Class(c_basic_object)
                       // -- m_name: the full path
                       m_segment: String;
                       m_c_parent_path: c_iso_path;

                       m_c_file_list: tStringList;
                       m_c_sub_path_list: tStringList;

                       Constructor create_iso_path(p_name, p_segment: String;
                           p_c_parent_path: c_iso_path);
                       function f_c_self: c_iso_path;

                       function f_file_count: Integer;
                       function f_c_file(p_file_index: Integer): c_iso_file;
                       function f_index_of_file(p_file_name: String): Integer;
                       function f_c_find_by_file(p_file_name: String): c_iso_file;
                       procedure add_file(p_file_name: String; p_c_file: c_iso_file);
                       function f_c_add_file(p_file_name: String;
                            p_iso_sector_number, p_iso_size: Integer): c_iso_file;
                       procedure display_file_list;

                       function f_sub_path_count: Integer;
                       function f_c_sub_path(p_sub_path_index: Integer): c_iso_path;
                       function f_c_find_sub_path_by_name(p_sub_path_name: String): c_iso_path;
                       procedure add_sub_path(p_sub_path_name: String; p_c_sub_path: c_iso_path);
                       function f_c_add_sub_path(p_sub_path_name, p_segment: String): c_iso_path;
                       function f_c_get_or_add_sub_path(p_sub_path_name, p_segment: String): c_iso_path;
                       procedure display_sub_path_list;

                       procedure display_path_and_file;

                       Destructor Destroy; Override;
                     end; // c_iso_path

  implementation
    uses SysUtils, Math
        , u_c_display
        , u_dir, u_strings
        ;

    // -- c_iso_file

    Constructor c_iso_file.create_iso_file(p_name: String;
        p_iso_sector_number, p_iso_size: Integer);
      begin
        Inherited create_basic_object(p_name);

        m_iso_sector_number:= p_iso_sector_number;
        m_iso_size:= p_iso_size;
      end; // create_iso_file

    function c_iso_file.f_display_file: String;
      begin
        Result:= Format('%-10s ', [m_name]);
      end; // f_display_file

    function c_iso_file.f_c_self: c_iso_file;
      begin
        Result:= Self;
      end; // f_c_self

    Destructor c_iso_file.Destroy;
      begin
        InHerited;
      end; // Destroy

    // -- c_iso_path

    Constructor c_iso_path.create_iso_path(p_name, p_segment: String;
        p_c_parent_path: c_iso_path);
      begin
        Inherited create_basic_object(p_name);
        m_segment:= p_segment;

        m_c_parent_path:= p_c_parent_path;

        m_c_file_list:= tStringList.Create;
        m_c_sub_path_list:= tStringList.Create;
      end; // create_iso_path

    function c_iso_path.f_c_self: c_iso_path;
      begin
        Result:= Self;
      end; // f_c_self
        
    // -- file handling

    function c_iso_path.f_file_count: Integer;
      begin
        Result:= m_c_file_list.Count;
      end; // f_file_count

    function c_iso_path.f_c_file(p_file_index: Integer): c_iso_file;
      begin
        Result:= c_iso_file(m_c_file_list.Objects[p_file_index]);
      end; //  f_c_file

    function c_iso_path.f_index_of_file(p_file_name: String): Integer;
      begin
        Result:= m_c_file_list.IndexOf(p_file_name);
      end; // f_index_of_file

    function c_iso_path.f_c_find_by_file(p_file_name: String): c_iso_file;
      var l_index_of: Integer;
      begin
        l_index_of:= f_index_of_file(p_file_name);
        if l_index_of< 0
          then Result:= Nil
          else Result:= c_iso_file(m_c_file_list.Objects[l_index_of]);
      end; // f_c_find_by_name

    procedure c_iso_path.add_file(p_file_name: String; p_c_file: c_iso_file);
      begin
        m_c_file_list.AddObject(p_file_name, p_c_file);
      end; // add_file

    function c_iso_path.f_c_add_file(p_file_name: String;
        p_iso_sector_number, p_iso_size: Integer): c_iso_file;
      begin
        Result:= f_c_file(m_c_file_list.AddObject(p_file_name,
            c_iso_file.create_iso_file(p_file_name, p_iso_sector_number, p_iso_size)));
      end; // f_c_add_file

    procedure c_iso_path.display_file_list;
      var l_file_index: Integer;
      begin
        display(m_name+ ' '+ IntToStr(f_file_count));

        for l_file_index:= 0 to f_file_count- 1 do
          display(f_c_file(l_file_index).f_display_file);
      end; // display_file_list

    // -- sub_path handling

    function c_iso_path.f_sub_path_count: Integer;
      begin
        Result:= m_c_sub_path_list.Count;
      end; // f_sub_path_count

    function c_iso_path.f_c_sub_path(p_sub_path_index: Integer): c_iso_path;
      begin
        Result:= c_iso_path(m_c_sub_path_list.Objects[p_sub_path_index]);
      end; //  f_c_sub_path

    function c_iso_path.f_c_find_sub_path_by_name(p_sub_path_name: String): c_iso_path;

      procedure find_by_sub_path_recursive(p_c_path: c_iso_path);
        var l_sub_path_index: Integer;
        begin
          with p_c_path do
          begin
            if m_name= p_sub_path_name
              then begin
                  Result:= f_c_self;
                  Exit;
                end;

            for l_sub_path_index:= 0 to f_sub_path_count- 1 do
            begin
              find_by_sub_path_recursive(f_c_sub_path(l_sub_path_index));
              if Result<> Nil
                then Break;
            end; // for l_path_index
          end; // with p_c_path
        end; // find_by_sub_path_recursive

      begin // f_c_find_sub_path_by_name
        Result:= Nil;
        find_by_sub_path_recursive(Self);
      end; // f_c_find_sub_path_by_name

    procedure c_iso_path.add_sub_path(p_sub_path_name: String; p_c_sub_path: c_iso_path);
      begin
        m_c_sub_path_list.AddObject(p_sub_path_name, p_c_sub_path);
      end; // add_sub_path

    function c_iso_path.f_c_add_sub_path(p_sub_path_name, p_segment: String): c_iso_path;
      begin
        Result:= c_iso_path.create_iso_path(p_sub_path_name, p_segment, Self);
        m_c_sub_path_list.AddObject(p_sub_path_name, Result);
      end; // f_c_add_sub_path

    function c_iso_path.f_c_get_or_add_sub_path(p_sub_path_name, p_segment: String): c_iso_path;
        // -- some applications use several calls to the same path (iso)
      begin
        Result:= f_c_find_sub_path_by_name(p_sub_path_name);
        if Result= Nil
          then Result:= f_c_add_sub_path(p_sub_path_name, p_segment);
      end; // f_c_get_or_add_sub_path

    procedure c_iso_path.display_sub_path_list;
      var l_sub_path_index: Integer;
      begin
        display(m_name+ ' '+ IntToStr(f_sub_path_count));

        for l_sub_path_index:= 0 to f_sub_path_count- 1 do
          with f_c_sub_path(l_sub_path_index) do
            display(m_name);
      end; // display_sub_path_list

    // -- both

    procedure c_iso_path.display_path_and_file;

      procedure display_recursive(p_level: Integer; p_c_path: c_iso_path);
        var l_sub_path_index: Integer;
            l_file_index: Integer;
            l_indentation: String;
        begin
          l_indentation:= f_spaces(p_level* 2);
          with p_c_path do
          begin
            display(l_indentation+ m_segment+ Format(' (%3d %4d) ', [f_sub_path_count, f_file_count]));

            // -- recurse
            for l_sub_path_index:= 0 to f_sub_path_count- 1 do
              display_recursive(p_level+ 1, f_c_sub_path(l_sub_path_index));

            // -- the files
            for l_file_index:= 0 to f_file_count- 1 do
              with f_c_file(l_file_index) do
                display(l_indentation+ '  -'+ m_name
                  + ' ['+ IntToStr(m_iso_sector_number)
                  + ' '+ IntToStr(m_iso_size)+ ']');

          end; // with p_c_path
        end; // display_recursive

      begin // display_path_and_file
        display_line;
        display(m_name);
        display_recursive(0, Self);
      end; // display_path_and_file

    Destructor c_iso_path.Destroy;
      var l_file_index: Integer;
          l_sub_path_index: Integer;
      begin
        for l_file_index:= 0 to f_file_count- 1 do
          f_c_file(l_file_index).Free;
        m_c_file_list.Free;

        for l_sub_path_index:= 0 to f_sub_path_count- 1 do
          f_c_sub_path(l_sub_path_index).Free;
        m_c_sub_path_list.Free;
        Inherited;
      end; // Destroy

    begin // u_c_path
    end. // u_c_path

