// 001 u_iso_definitions
// 26 sep 2004

// -- (C) John COLIBRI 2006
// -- documentation: http://www.jcolibri.com

// -- Delphi definition of the ISO records

(*$r+*)

unit u_iso_definitions;
  interface

    type TBothEndianWord= packed record
                            LittleEndian, BigEndian: Word;
                          end;
         TBothEndianDWord= packed record
                             LittleEndian, BigEndian: LongWord;
                           end;

         TVolumeDateTime= packed record
                            Year: array[0..3] of Char;
                            Month: array[0..1] of Char;
                            Day: array[0..1] of Char;
                            Hour: array[0..1] of Char;
                            Minute: array[0..1] of Char;
                            Second: array[0..1] of Char;
                            MSeconds: array[0..1] of Char;
                            GMTOffset: Byte;
                          end;

         TDirectoryDateTime= packed record
                               Year: Byte; // since 1900
                               Month: Byte;
                               Day: Byte;
                               Hour: Byte;
                               Minute: Byte;
                               Second: Byte;
                               GMTOffset: Byte; // in 15 minutes steps
                             end;

         TRootDirectoryRecord= packed record
                                 // -- 0000h
                                 LengthOfDirectoryRecord: Byte;
                                 // -- 0001h
                                 ExtendedAttributeRecordLength: Byte;
                                 // -- 0002h
                                 LocationOfExtent: TBothEndianDWord;
                                 // -- 000Ah
                                 DataLength: TBothEndianDWord;
                                 // -- 0012h
                                 RecordingDateAndTime: TDirectoryDateTime;
                                 // -- 0019h
                                 FileFlags: Byte;
                                 // -- bit     value
                                 // -- ------------------------------------------
                                 // -- 0 (LS)  0 for a norma1 file, 1 for a hidden file
                                 // -- 1       0 for a file, 1 for a directory
                                 // -- 2       0 [1 for an associated file]
                                 // -- 3       0 [1 for record format specified]
                                 // -- 4       0 [1 for permissions specified]
                                 // -- 5       0
                                 // -- 6       0
                                 // -- 7 (MS)  0 [1 if not the final record for the file]

                                 // -- 001Ah :
                                 FileUnitSize: Byte;
                                 // -- 001Bh :
                                 InterleaveGapSize: Byte;
                                 // -- 001Ch
                                 VolumeSequenceNumber: TBothEndianWord;
                                 // -- 0020h
                                 LengthOfFileIdentifier: Byte; // = 1
                                 // -- 0021h
                                 FileIdentifier: Byte; // = 0
                               end;

         t_directory_entry= packed record
                              LengthOfDirectoryRecord: Byte;
                              ExtendedAttributeRecordLength: Byte;
                              LocationOfExtent: TBothEndianDWord;
                              DataLength: TBothEndianDWord;
                              RecordingDateAndTime: TDirectoryDateTime;
                              FileFlags: Byte;
                              // -- bit     value
                              // -- ------------------------------------------
                              // -- 0 (LS)  0 for a norma1 file, 1 for a hidden file
                              // -- 1       0 for a file, 1 for a directory
                              // -- 2       0 [1 for an associated file]
                              // -- 3       0 [1 for record format specified]
                              // -- 4       0 [1 for permissions specified]
                              // -- 5       0
                              // -- 6       0
                              // -- 7 (MS)  0 [1 if not the final record for the file]
                              FileUnitSize: Byte;
                              InterleaveGapSize: Byte;
                              VolumeSequenceNumber: TBothEndianWord;
                              LengthOfFileIdentifier: Byte;
                              // -- followed by FileIdentifier and padding bytes
                            end; // t_directory_entry
         t_pt_directory_entry= ^t_directory_entry;

         TPathTableRecord= packed record
                             LengthOfDirectoryIdentifier: Byte;
                             ExtendedAttributeRecordLength: Byte;
                             LocationOfExtent: Cardinal;
                             ParentDirectoryNumber: Word;
                             // -- followed by DirectoryIdentifier with [LengthOfDirectoryIdentifier] bytes
                             // -- followed by padding byte, if [LengthOfDirectoryIdentifier] is odd
                           end;
         PPathTableRecord= ^TPathTableRecord;

         // -- Primary Volume Descriptor
         t_primary_descriptor= packed record
                                 // -- 0001h
                                 StandardIdentifier: array[0..4] of Char;
                                 // -- 0006h
                                 VolumeDescriptorVersion: Byte;
                                 // -- 0007h
                                 unused: Byte;
                                 // -- 0007h
                                 SystemIdentifier: array[0..31] of Char;
                                 // -- 0027h
                                 VolumeIdentifier: array[0..31] of Char;
                                 // -- 0047h
                                 Unused2: array[0..7] of Byte;

                                 VolumeSpaceSize: TBothEndianDWord;
                                 Unused3: array[0..31] of Byte;
                                 VolumeSetSize: TBothEndianWord;
                                 VolumeSequenceNumber: TBothEndianWord;
                                 LogicalBlockSize: TBothEndianWord;
                                 PathTableSize: TBothEndianDWord;
                                 LocationOfTypeLPathTable: LongWord;
                                 LocationOfOptionalTypeLPathTable: LongWord;
                                 LocationOfTypeMPathTable: LongWord;
                                 LocationOfOptionalTypeMPathTable: LongWord;
                                 RootDirectory: TRootDirectoryRecord;

                                 VolumeSetIdentifier: array[0..127] of Char;
                                 PublisherIdentifier: array[0..127] of Char;
                                 DataPreparerIdentifier: array[0..127] of Char;
                                 ApplicationIdentifier: array[0..127] of Char;
                                 CopyrightFileIdentifier: array[0..36] of Char;
                                 AbstractFileIdentifier: array[0..36] of Char;
                                 BibliographicFileIdentifier: array[0..36] of Char;
                                 VolumeCreationDateAndTime: TVolumeDateTime;
                                 VolumeModificationDateAndTime: TVolumeDateTime;
                                 VolumeExpirationDateAndTime: TVolumeDateTime;
                                 VolumeEffectiveDateAndTime: TVolumeDateTime;
                                 FileStructureVersion: Byte;
                                 ReservedForFutureStandardization: Byte;

                                 // -- the remaining bytes
                                 ApplicationUse: array[0..511] of Byte;
                                 ReservedForFutureStandardization2: array[0..652] of Byte;
                               end; // t_primary_descriptor

         // -- Supplementary Volume Descriptor
         t_supplementary_descriptor= packed record
                                       // -- 0001h
                                       StandardIdentifier: array[0..4] of Char;
                                       // -- 0006h
                                       VolumeDescriptorVersion: Byte;
                                       // -- 0007h
                                       VolumeFlags: Byte;
                                       // -- 0007h
                                       SystemIdentifier: array[0..31] of Char;
                                       // -- 0027h
                                       VolumeIdentifier: array[0..31] of Char;
                                       // -- 0047h
                                       Unused2: array[0..7] of Byte;
                                       VolumeSpaceSize: TBothEndianDWord;
                                       EscapeSequences: array[0..31] of Byte;
                                       VolumeSetSize: TBothEndianWord;
                                       VolumeSequenceNumber: TBothEndianWord;
                                       LogicalBlockSize: TBothEndianWord;
                                       PathTableSize: TBothEndianDWord;
                                       LocationOfTypeLPathTable: LongWord;
                                       LocationOfOptionalTypeLPathTable: LongWord;
                                       LocationOfTypeMPathTable: LongWord;
                                       LocationOfOptionalTypeMPathTable: LongWord;
                                       RootDirectory: TRootDirectoryRecord;
                                       VolumeSetIdentifier: array[0..127] of Char;
                                       PublisherIdentifier: array[0..127] of Char;
                                       DataPreparerIdentifier: array[0..127] of Char;
                                       ApplicationIdentifier: array[0..127] of Char;
                                       CopyrightFileIdentifier: array[0..36] of Char;
                                       AbstractFileIdentifier: array[0..36] of Char;
                                       BibliographicFileIdentifier: array[0..36] of Char;
                                       VolumeCreationDateAndTime: TVolumeDateTime;
                                       VolumeModificationDateAndTime: TVolumeDateTime;
                                       VolumeExpirationDateAndTime: TVolumeDateTime;
                                       VolumeEffectiveDateAndTime: TVolumeDateTime;
                                       FileStructureVersion: Byte;
                                       ReservedForFutureStandardization: Byte;

                                       ApplicationUse: array[0..511] of Byte;
                                       ReservedForFutureStandardization2
                                           : array[0..652] of Byte;
                                     end; // t_supplementary_descriptor

         // -- Boot Record Volume Descriptor
         t_boot_record_descriptor= packed record
                                     StandardIdentifier: array[0..4] of Char;
                                     VersionOfDescriptor: Byte;
                                     BootSystemIdentifier: array[0..31] of Char;
                                     BootIdentifier: array[0..31] of Char;
                                     BootCatalogPointer: LongWord;
                                     Unused: array[0..1972] of Byte;
                                   end; // t_boot_record_descriptor

    const // -- Volume Descriptor Types
          vdtBR= $00; // Boot Record
          vdtPVD= $01; // Primary Volume Descriptor
          vdtSVD= $02; // Supplementary Volume Descriptor
          vdtVDST= $ff; // Volume Descriptor Set Terminator

    type // this is a 2048 structure : 1 byte for the type, the rest in the descriptor
         TVolumeDescriptor= packed record
                                     case DescriptorType: Byte of
                                       vdtPVD: (Primary: t_primary_descriptor);
                                       vdtBR: (BootRecord: t_boot_record_descriptor);
                                       vdtSVD: (Supplementary: t_supplementary_descriptor);
                                   end;

  implementation
    uses SysUtils;

    end.


